@@ -709,9 +709,9 @@ bool ShaderGenerator::hasShaderBasedTechnique(const String& materialName,
 //-----------------------------------------------------------------------------
 static bool hasFixedFunctionPass(Technique* tech)
 {
-    for (unsigned short i=0; i < tech->getNumPasses(); ++i)
+    for (const auto& p : tech->getPasses())
     {
-        if (!tech->getPass(i)->isProgrammable())
+        if (!p->isProgrammable())
         {
             return true;
         }
@@ -724,7 +724,7 @@ static Technique* findSourceTechnique(const Material& mat, const String& srcTech
     // Find the source technique
     for (auto *t : mat.getTechniques())
     {
-        if (t->getSchemeName() == srcTechniqueSchemeName &&
+        if (t->getSchemeName() == srcTechniqueSchemeName && t->isSupported() &&
             (hasFixedFunctionPass(t) || overProgrammable))
         {
             return t;
@@ -894,7 +894,7 @@ bool ShaderGenerator::removeAllShaderBasedTechniques(const String& materialName,
     return true;
 }
 
-bool ShaderGenerator::cloneShaderBasedTechniques(const Material& srcMat, Material& dstMat)
+bool ShaderGenerator::cloneShaderBasedTechniques(Material& srcMat, Material& dstMat)
 {
     if(&srcMat == &dstMat) return true; // nothing to do
 
@@ -938,6 +938,8 @@ bool ShaderGenerator::cloneShaderBasedTechniques(const Material& srcMat, Materia
     // Clone the render states from source to destination
     //
 
+    srcMat.load(); // ensure supported techniques are loaded
+
     // Check if RTSS techniques exist in the source material
     if (itSrcMatEntry != mMaterialEntriesMap.end())
     {
@@ -954,7 +956,7 @@ bool ShaderGenerator::cloneShaderBasedTechniques(const Material& srcMat, Materia
             if (createShaderBasedTechnique(dstMat, srcFromTechniqueScheme, srcToTechniqueScheme))
             {
                 //check for custom render states in the source material
-                unsigned short passCount =  t->getSourceTechnique()->getNumPasses();
+                unsigned short passCount = t->getSourceTechnique()->getNumPasses();
                 for(unsigned short pi = 0 ; pi < passCount ; ++pi)
                 {
                     if (t->hasRenderState(pi))
@@ -963,13 +965,8 @@ bool ShaderGenerator::cloneShaderBasedTechniques(const Material& srcMat, Materia
                         RenderState* srcRenderState = t->getRenderState(pi);
                         RenderState* dstRenderState = getRenderState(srcToTechniqueScheme, dstMat, pi);
 
-                        const SubRenderStateList& srcSubRenderState =
-                            srcRenderState->getSubRenderStates();
-
-                        SubRenderStateList::const_iterator itSubState = srcSubRenderState.begin(), itSubStateEnd = srcSubRenderState.end();
-                        for(;itSubState != itSubStateEnd ; ++itSubState)
+                        for(SubRenderState* srcSubState : srcRenderState->getSubRenderStates())
                         {
-                            SubRenderState* srcSubState = *itSubState;
                             SubRenderState* dstSubState = createSubRenderState(srcSubState->getType());
                             (*dstSubState) = (*srcSubState);
                             dstRenderState->addTemplateSubRenderState(dstSubState);
