@@ -28,6 +28,12 @@
 #ifndef __SampleBrowser_H__
 #define __SampleBrowser_H__
 
+#ifdef HAVE_IMGUI
+#include "OgreImGuiOverlay.h"
+#endif
+
+#include "OgreOverlayManager.h"
+#include "OgreRenderTargetListener.h"
 #include "SampleContext.h"
 #include "SamplePlugin.h"
 #include "OgreTrays.h"
@@ -66,7 +72,7 @@ namespace OgreBites
       | The OGRE Sample Browser. Features a menu accessible from all samples,
       | dynamic configuration, resource reloading, node labeling, and more.
       =============================================================================*/
-    class SampleBrowser : public SampleContext, public TrayListener
+    class SampleBrowser : public SampleContext, public TrayListener, public Ogre::RenderTargetListener
     {
 #ifdef OGRE_STATIC_LIB
         typedef std::map<Ogre::String, SamplePlugin*> PluginMap;
@@ -88,7 +94,6 @@ namespace OgreBites
             mSampleSlider = 0;
             mTitleLabel = 0;
             mDescBox = 0;
-            mRendererMenu = 0;
             mCarouselPlace = 0.0f;
 
 #if OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
@@ -138,8 +143,6 @@ namespace OgreBites
 
                 try
                 {
-                    if(dynamic_cast<SdkSample*>(s))
-                        s->_setupTrays(mWindow);
                     SampleContext::runSample(s);
                 }
                 catch (Ogre::Exception& e)   // if failed to start, show error and fall back to menu
@@ -285,8 +288,40 @@ namespace OgreBites
                     catch (Ogre::Exception&) {}
                 }
             }
+#ifdef HAVE_IMGUI
             else if (b->getName() == "Configure")   // enter configuration screen
             {
+                mOwnsImGuiOverlay = !Ogre::OverlayManager::getSingleton().getByName("ImGuiOverlay");
+                auto imguiOverlay = initialiseImGui();
+                imguiOverlay->addFont("SdkTrays/Caption", "Essential");
+                imguiOverlay->setZOrder(300);
+                imguiOverlay->show();
+
+                if(mOwnsImGuiOverlay)
+                {
+                    ImGui::GetStyle().FrameRounding = 8;
+                    ImVec4* colors = ImGui::GetStyle().Colors;
+                    colors[ImGuiCol_FrameBg]                = ImVec4(0.16f, 0.48f, 0.2f, 0.54f);
+                    colors[ImGuiCol_FrameBgHovered]         = ImVec4(0.26f, 0.98f, 0.2f, 0.40f);
+                    colors[ImGuiCol_FrameBgActive]          = ImVec4(0.26f, 0.98f, 0.2f, 0.67f);
+                    colors[ImGuiCol_TitleBgActive]          = ImVec4(0.16f, 0.48f, 0.2f, 1.00f);
+                    colors[ImGuiCol_CheckMark]              = ImVec4(0.26f, 0.98f, 0.2f, 1.00f);
+                    colors[ImGuiCol_SliderGrab]             = ImVec4(0.24f, 0.88f, 0.2f, 1.00f);
+                    colors[ImGuiCol_SliderGrabActive]       = ImVec4(0.26f, 0.98f, 0.2f, 1.00f);
+                    colors[ImGuiCol_Button]                 = ImVec4(0.26f, 0.98f, 0.2f, 0.40f);
+                    colors[ImGuiCol_ButtonHovered]          = ImVec4(0.26f, 0.98f, 0.2f, 1.00f);
+                    colors[ImGuiCol_ButtonActive]           = ImVec4(0.06f, 0.98f, 0.2f, 1.00f);
+                    colors[ImGuiCol_Header]                 = ImVec4(0.26f, 0.98f, 0.2f, 0.31f);
+                    colors[ImGuiCol_HeaderHovered]          = ImVec4(0.26f, 0.98f, 0.2f, 0.80f);
+                    colors[ImGuiCol_HeaderActive]           = ImVec4(0.26f, 0.98f, 0.2f, 1.00f);
+                    colors[ImGuiCol_ResizeGrip]             = ImVec4(0.26f, 0.98f, 0.2f, 0.20f);
+                    colors[ImGuiCol_ResizeGripHovered]      = ImVec4(0.26f, 0.98f, 0.2f, 0.67f);
+                    colors[ImGuiCol_ResizeGripActive]       = ImVec4(0.26f, 0.98f, 0.2f, 0.95f);
+                }
+
+                mWindow->addListener(this);
+                mInputListenerChain = TouchAgnosticInputListenerChain(mWindow, {this, mTrayMgr, getImGuiInputListener()});
+
                 mTrayMgr->removeWidgetFromTray("StartStop");
                 mTrayMgr->removeWidgetFromTray("Configure");
 #if OGRE_PLATFORM != OGRE_PLATFORM_ANDROID
@@ -311,20 +346,17 @@ namespace OgreBites
                     mTrayMgr->removeWidgetFromTray(TL_LEFT, 0);
                 }
 
-                mTrayMgr->moveWidgetToTray("ConfigLabel", TL_LEFT);
-                mTrayMgr->moveWidgetToTray(mRendererMenu, TL_LEFT);
-                mTrayMgr->moveWidgetToTray("ConfigSeparator", TL_LEFT);
-
-                mRendererMenu->selectItem(mRoot->getRenderSystem()->getName());
-
                 windowResized(mWindow);
             }
             else if (b->getName() == "Back")   // leave configuration screen
             {
-                while (mTrayMgr->getWidgets(mRendererMenu->getTrayLocation()).size() > 3)
-                {
-                    mTrayMgr->destroyWidget(mRendererMenu->getTrayLocation(), 3);
-                }
+                if(mOwnsImGuiOverlay)
+                    Ogre::OverlayManager::getSingleton().destroy("ImGuiOverlay"); // bring down overly to avoid interfering with samples
+                else
+                    Ogre::ImGuiOverlay::NewFrame(); // clear dialog
+
+                mInputListenerChain = TouchAgnosticInputListenerChain(mWindow, {this, mTrayMgr});
+                mWindow->removeListener(this);
 
                 while (!mTrayMgr->getWidgets(TL_NONE).empty())
                 {
@@ -333,9 +365,6 @@ namespace OgreBites
 
                 mTrayMgr->removeWidgetFromTray("Apply");
                 mTrayMgr->removeWidgetFromTray("Back");
-                mTrayMgr->removeWidgetFromTray("ConfigLabel");
-                mTrayMgr->removeWidgetFromTray(mRendererMenu);
-                mTrayMgr->removeWidgetFromTray("ConfigSeparator");
 
                 mTrayMgr->moveWidgetToTray("StartStop", TL_RIGHT);
 #if OGRE_PLATFORM != OGRE_PLATFORM_ANDROID
@@ -349,30 +378,10 @@ namespace OgreBites
                 mCarouselPlace += CAROUSEL_REDRAW_EPS;  // force redraw
                 windowResized(mWindow);
             }
+#endif
             else if (b->getName() == "Apply")   // apply any changes made in the configuration screen
             {
-                bool reset = false;
-
-                auto options =
-                    mRoot->getRenderSystemByName(mRendererMenu->getSelectedItem())->getConfigOptions();
-
-                Ogre::NameValuePairList newOptions;
-
-                // collect new settings and decide if a reset is needed
-
-                if (mRendererMenu->getSelectedItem() != mRoot->getRenderSystem()->getName()) {
-                    reset = true;
-                }
-
-                for (unsigned int i = 3; i < mTrayMgr->getWidgets(mRendererMenu->getTrayLocation()).size(); i++)
-                {
-                    SelectMenu* menu = (SelectMenu*)mTrayMgr->getWidgets(mRendererMenu->getTrayLocation())[i];
-                    if (menu->getSelectedItem() != options[menu->getCaption()].currentValue) reset = true;
-                    newOptions[menu->getCaption()] = menu->getSelectedItem();
-                }
-
-                // reset with new settings if necessary
-                if (reset) reconfigure(mRendererMenu->getSelectedItem(), newOptions);
+                reconfigure(mNextRenderer);
             }
             else
             {
@@ -380,6 +389,24 @@ namespace OgreBites
             }
         }
 
+        void preViewportUpdate(const Ogre::RenderTargetViewportEvent& evt) override
+        {
+#ifdef HAVE_IMGUI
+            Ogre::ImGuiOverlay::NewFrame();
+
+            auto flags = ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoCollapse |
+                         ImGuiWindowFlags_NoSavedSettings | ImGuiWindowFlags_NoMove;
+            auto center = ImGui::GetMainViewport()->GetCenter();
+            if(mWindow->getWidth() <= 1280)
+                ImGui::SetNextWindowPos(ImVec2(0, center.y), ImGuiCond_Always, ImVec2(0.f, 0.5f));
+            else
+                ImGui::SetNextWindowPos(center, ImGuiCond_Always, ImVec2(0.5f, 0.5f));
+            ImGui::Begin("Configuration", NULL, flags);
+            Ogre::DrawRenderingSettings(mNextRenderer);
+            ImGui::End();
+#endif
+        }
+
         /*-----------------------------------------------------------------------------
           | Handles menu item selection changes.
           -----------------------------------------------------------------------------*/
@@ -458,36 +485,6 @@ namespace OgreBites
                 if (mCurrentSample != s) ((Button*)mTrayMgr->getWidget("StartStop"))->setCaption("Start Sample");
                 else ((Button*)mTrayMgr->getWidget("StartStop"))->setCaption("Stop Sample");
             }
-            else if (menu == mRendererMenu)    // renderer selected, so update all settings
-            {
-                while (mTrayMgr->getWidgets(mRendererMenu->getTrayLocation()).size() > 3)
-                {
-                    mTrayMgr->destroyWidget(mRendererMenu->getTrayLocation(), 3);
-                }
-
-                auto options = mRoot->getRenderSystemByName(menu->getSelectedItem())->getConfigOptions();
-
-                unsigned int i = 0;
-
-                // create all the config option select menus
-                for (Ogre::ConfigOptionMap::iterator it = options.begin(); it != options.end(); it++)
-                {
-                    i++;
-                    SelectMenu* optionMenu = mTrayMgr->createLongSelectMenu
-                        (TL_LEFT, "ConfigOption" + Ogre::StringConverter::toString(i), it->first, 450, 240, 10);
-                    optionMenu->setItems(it->second.possibleValues);
-
-                    // if the current config value is not in the menu, add it
-                    if(optionMenu->containsItem(it->second.currentValue) == false)
-                    {
-                        optionMenu->addItem(it->second.currentValue);
-                    }
-
-                    optionMenu->selectItem(it->second.currentValue);
-                }
-
-                windowResized(mWindow);
-            }
         }
 
         /*-----------------------------------------------------------------------------
@@ -636,16 +633,8 @@ namespace OgreBites
                 }
             }
 
-            if (isCurrentSamplePaused()) return mTrayMgr->mousePressed(evt);
-
-            return SampleContext::mousePressed(evt);
-        }
-
-        // convert and redirect
-        bool touchPressed(const TouchFingerEvent& evt) override {
-            MouseButtonEvent e;
-            e.button = BUTTON_LEFT;
-            return mousePressed(e);
+            if(!isCurrentSamplePaused()) return true; // stop propagation
+            return false;
         }
 
         bool buttonPressed(const ButtonEvent& evt) override
@@ -674,17 +663,9 @@ namespace OgreBites
           | Extends pointerReleased to inject mouse release into tray manager.
           -----------------------------------------------------------------------------*/
         bool mouseReleased(const MouseButtonEvent& evt) override
-         {
-            if (isCurrentSamplePaused()) return mTrayMgr->mouseReleased(evt);
-
-            return SampleContext::mouseReleased(evt);
-        }
-
-        // convert and redirect
-        bool touchReleased(const TouchFingerEvent& evt) override {
-            MouseButtonEvent e;
-            e.button = BUTTON_LEFT;
-            return mouseReleased(e);
+        {
+            if(!isCurrentSamplePaused()) return true; // stop propagation
+            return false;
         }
 
         /*-----------------------------------------------------------------------------
@@ -693,19 +674,8 @@ namespace OgreBites
           -----------------------------------------------------------------------------*/
         bool mouseMoved(const MouseMotionEvent& evt) override
         {
-            if (isCurrentSamplePaused()) return mTrayMgr->mouseMoved(evt);
-
-            return SampleContext::mouseMoved(evt);
-        }
-
-        // convert and redirect
-        bool touchMoved(const TouchFingerEvent& evt) override {
-            MouseMotionEvent e;
-            e.x = evt.x * mWindow->getWidth();
-            e.y = evt.y * mWindow->getHeight();
-            e.xrel = evt.dx * mWindow->getWidth();
-            e.yrel = evt.dy * mWindow->getHeight();
-            return mouseMoved(e);
+            if(!isCurrentSamplePaused()) return true; // stop propagation
+            return false;
         }
 
         //TODO: Handle iOS and Android.
@@ -716,8 +686,9 @@ namespace OgreBites
             if(mTrayMgr->mouseWheelRolled(evt))
                 return true;
 
-            if (isCurrentSamplePaused() && mTitleLabel->getTrayLocation() != TL_NONE &&
-                mSampleMenu->getNumItems() != 0)
+            if(!isCurrentSamplePaused()) return true; // stop propagation
+
+            if (mTitleLabel->getTrayLocation() != TL_NONE && mSampleMenu->getNumItems() != 0)
             {
                 int newIndex = mSampleMenu->getSelectionIndex() - evt.y / Ogre::Math::Abs(evt.y);
                 mSampleMenu->selectItem(Ogre::Math::Clamp<int>(newIndex, 0, mSampleMenu->getNumItems() - 1));
@@ -763,7 +734,9 @@ namespace OgreBites
         {
             ApplicationContext::setup();
             mWindow = getRenderWindow();
-            addInputListener(this);
+
+            mInputListenerChain = TouchAgnosticInputListenerChain(mWindow, {this, mTrayMgr});
+            addInputListener(&mInputListenerChain);
             if(mGrabInput) setWindowGrab();
             else mTrayMgr->hideCursor();
 #ifdef OGRE_STATIC_LIB
@@ -984,7 +957,7 @@ namespace OgreBites
 #if (OGRE_PLATFORM != OGRE_PLATFORM_WINRT) && (OGRE_PLATFORM != OGRE_PLATFORM_ANDROID)
             mTrayMgr->createButton(TL_RIGHT, "UnloadReload", mLoadedSamples.empty() ? "Reload Samples" : "Unload Samples");
 #endif
-#if (OGRE_PLATFORM != OGRE_PLATFORM_WINRT)
+#if (OGRE_PLATFORM != OGRE_PLATFORM_WINRT) && defined(HAVE_IMGUI)
             mTrayMgr->createButton(TL_RIGHT, "Configure", "Configure");
 #endif
 #if (OGRE_PLATFORM != OGRE_PLATFORM_ANDROID)
@@ -1010,20 +983,6 @@ namespace OgreBites
             mTrayMgr->createButton(TL_NONE, "Apply", "Apply Changes");
             mTrayMgr->createButton(TL_NONE, "Back", "Go Back");
 
-            // create configuration screen label and renderer menu
-            mTrayMgr->createLabel(TL_NONE, "ConfigLabel", "Configuration");
-            mRendererMenu = mTrayMgr->createLongSelectMenu(TL_NONE, "RendererMenu", "Render System", 450, 240, 10);
-            mTrayMgr->createSeparator(TL_NONE, "ConfigSeparator");
-
-            // populate render system names
-            Ogre::StringVector rsNames;
-            Ogre::RenderSystemList rsList = mRoot->getAvailableRenderers();
-            for (unsigned int i = 0; i < rsList.size(); i++)
-            {
-                rsNames.push_back(rsList[i]->getName());
-            }
-            mRendererMenu->setItems(rsNames);
-
             populateSampleMenus();
         }
 
@@ -1080,7 +1039,7 @@ namespace OgreBites
         /*-----------------------------------------------------------------------------
           | Extends reconfigure to save the view and the index of last sample run.
           -----------------------------------------------------------------------------*/
-        void reconfigure(const Ogre::String& renderer, Ogre::NameValuePairList& options) override
+        void reconfigure(const Ogre::String& renderer)
         {
             mLastViewCategory = mCategoryMenu->getSelectionIndex();
             mLastViewTitle = mSampleMenu->getSelectionIndex();
@@ -1097,7 +1056,7 @@ namespace OgreBites
                 }
             }
 
-            SampleContext::reconfigure(renderer, options);
+            SampleContext::reconfigure(renderer);
         }
     public:
         /*-----------------------------------------------------------------------------
@@ -1123,7 +1082,6 @@ namespace OgreBites
             mSampleSlider = 0;
             mTitleLabel = 0;
             mDescBox = 0;
-            mRendererMenu = 0;
             mHiddenOverlays.clear();
             mThumbs.clear();
             mCarouselPlace = 0;
@@ -1177,7 +1135,6 @@ namespace OgreBites
         Slider* mSampleSlider;                         // sample slider bar
         Label* mTitleLabel;                            // sample title label
         TextBox* mDescBox;                             // sample description box
-        SelectMenu* mRendererMenu;                     // render system selection menu
         std::vector<Ogre::Overlay*> mHiddenOverlays;   // sample overlays hidden for pausing
         std::vector<Ogre::OverlayContainer*> mThumbs;  // sample thumbnails
         Ogre::Real mCarouselPlace;                     // current state of carousel
@@ -1185,12 +1142,14 @@ namespace OgreBites
         int mLastViewCategory;                         // last sample category viewed
         int mLastSampleIndex;                          // index of last sample running
         int mStartSampleIndex;                         // directly starts the sample with the given index
+        TouchAgnosticInputListenerChain mInputListenerChain;
     public:
 #if OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
         SampleBrowserGestureView *mGestureView;
 #endif
         bool mIsShuttingDown;
         bool mGrabInput;
+        bool mOwnsImGuiOverlay;
     };
 }
 
