@@ -636,20 +636,6 @@ protected:
 
 };
 
-//------------------------------------------------------------------------------
-/** Tests billboard texture coordinates */
-class _OgreSampleClassExport PlayPen_BillboardTextureCoords : public VisualTest
-{
-public:
-
-    PlayPen_BillboardTextureCoords();
-
-protected:
-
-    void setupContent() override;
-
-};
-
 //------------------------------------------------------------------------------
 /** Tests particles with an RTT reflection */
 class _OgreSampleClassExport PlayPen_ReflectedBillboards : public VisualTest
